@@ -16,7 +16,7 @@ use vars qw(@ISA @EXPORT %dh);
 	    &compat &addsubstvar &delsubstvar &excludefile &package_arch
 	    &is_udeb &udeb_filename &debhelper_script_subst &escape_shell);
 
-my $max_compat=5;
+my $max_compat=6;
 
 sub init {
 	# If DH_OPTIONS is set, prepend it @ARGV.
@@ -55,7 +55,7 @@ sub init {
 		foreach (@{$dh{EXCLUDE}}) {
 			my $x=$_;
 			$x=escape_shell($x);
-			$x=~s/\./\\./g;
+			$x=~s/\./\\\\./g;
 			$dh{EXCLUDE_FIND}.="-regex .\\*$x.\\* -or ";
 		}
 		$dh{EXCLUDE_FIND}=~s/ -or $//;
@@ -104,7 +104,7 @@ sub init {
 	# If no error handling function was specified, just propigate
 	# errors out.
 	if (! exists $dh{ERROR_HANDLER} || ! defined $dh{ERROR_HANDLER}) {
-		$dh{ERROR_HANDLER}='exit $?';
+		$dh{ERROR_HANDLER}='exit \$?';
 	}
 }
 
@@ -271,8 +271,8 @@ sub dirname {
 			}
 		}
 
-		if ($c < 3 && ! $warned_compat) {
-			warning("Compatibility levels before 3 are deprecated.");
+		if ($c < 4 && ! $warned_compat) {
+			warning("Compatibility levels before 4 are deprecated.");
 			$warned_compat=1;
 		}
 	
@@ -320,18 +320,22 @@ sub pkgfile {
 		$filename="$dh{NAME}.$filename";
 	}
 	
-	if (-f "debian/$package.$filename.".buildarch()) {
-		return "debian/$package.$filename.".buildarch();
+	my @try=("debian/$package.$filename.".buildarch(),
+		 "debian/$package.$filename");
+	if ($package eq $dh{MAINPACKAGE}) {
+		push @try, "debian/$filename";
 	}
-	elsif (-f "debian/$package.$filename") {
-		return "debian/$package.$filename";
-	}
-	elsif ($package eq $dh{MAINPACKAGE} && -f "debian/$filename") {
-		return "debian/$filename";
-	}
-	else {
-		return "";
+	
+	foreach my $file (@try) {
+		if (-f $file &&
+		    (! $dh{IGNORE} || ! exists $dh{IGNORE}->{$file})) {
+			return $file;
+		}
+
 	}
+
+	return "";
+
 }
 
 # Pass it a name of a binary package, it returns the name to prefix to files
@@ -405,7 +409,7 @@ sub autoscript {
 	my $filename=shift;
 	my $sed=shift || "";
 
-	# This is the file we will append to.
+	# This is the file we will modify.
 	my $outfile="debian/".pkgext($package)."$script.debhelper";
 
 	# Figure out what shell script snippet to use.
@@ -423,9 +427,20 @@ sub autoscript {
 		}
 	}
 
-	complex_doit("echo \"# Automatically added by ".basename($0)."\">> $outfile");
-	complex_doit("sed \"$sed\" $infile >> $outfile");
-	complex_doit("echo '# End automatically added section' >> $outfile");
+	if (-e $outfile && ($script eq 'postrm' || $script eq 'prerm')
+	   && !compat(5)) {
+		# Add fragments to top so they run in reverse order when removing.
+		complex_doit("echo \"# Automatically added by ".basename($0)."\"> $outfile.new");
+		complex_doit("sed \"$sed\" $infile >> $outfile.new");
+		complex_doit("echo '# End automatically added section' >> $outfile.new");
+		complex_doit("cat $outfile >> $outfile.new");
+		complex_doit("mv $outfile.new $outfile");
+	}
+	else {
+		complex_doit("echo \"# Automatically added by ".basename($0)."\">> $outfile");
+		complex_doit("sed \"$sed\" $infile >> $outfile");
+		complex_doit("echo '# End automatically added section' >> $outfile");
+	}
 }
 
 # Removes a whole substvar line.
@@ -553,12 +568,24 @@ sub excludefile {
 	sub buildarch {
   		return $arch if defined $arch;
 
-		$arch=`dpkg-architecture -qDEB_HOST_ARCH 2>/dev/null` || error($!);
+		$arch=`dpkg-architecture -qDEB_HOST_ARCH 2>/dev/null` || error("dpkg-architecture failed");
 		chomp $arch;
 		return $arch;
 	}
 }
 
+# Passed an arch and a list of arches to match against, returns true if matched
+sub samearch {
+	my $arch=shift;
+	my @archlist=split(/\s+/,shift);
+
+	foreach my $a (@archlist) {
+		system("dpkg-architecture", "-a$arch", "-i$a") == 0 && return 1;
+	}
+
+	return 0;
+}
+
 # Returns a list of packages in the control file.
 # Must pass "arch" or "indep" or "same" to specify arch-dependant or
 # -independant or same arch packages. If nothing is specified, returns all
@@ -604,7 +631,7 @@ sub getpackages {
 		if (/^Architecture:\s*(.*)/) {
 			$arch=$1;
 		}
-		if (/^X[BC]*-Package-Type:\s*(.*)/) {
+		if (/^(?:X[BC]*-)?Package-Type:\s*(.*)/) {
 			$package_type=$1;
 		}
 		
@@ -616,7 +643,7 @@ sub getpackages {
 			if ($package &&
 			    (($type eq 'indep' && $arch eq 'all') ||
 			     ($type eq 'arch' && $arch ne 'all') ||
-			     ($type eq 'same' && ($arch eq 'any' || $arch =~ /\b$buildarch\b/)) ||
+			     ($type eq 'same' && ($arch eq 'any' || samearch($buildarch, $arch))) ||
 			     ! $type)) {
 				push @list, $package;
 				$package="";
@@ -633,6 +660,10 @@ sub getpackages {
 sub package_arch {
 	my $package=shift;
 	
+	if (! exists $package_arches{$package}) {
+		warning "package $package is not in control info";
+		return buildarch();
+	}
 	return $package_arches{$package} eq 'all' ? "all" : buildarch();
 }
 
@@ -640,6 +671,10 @@ sub package_arch {
 sub is_udeb {
 	my $package=shift;
 	
+	if (! exists $package_types{$package}) {
+		warning "package $package is not in control info";
+		return 0;
+	}
 	return $package_types{$package} eq 'udeb';
 }
 
